#coding=utf-8
from buildz import xf, pyz
from buildz.xf import g as xg
import json
import builtins
typez = builtins.type
class IOCError(Exception):
    def __init__(self, msg):
        super().__init__(msg)

pass
class Base:
    def update_maps(self, maps, src, replace=1):
        xf.deep_update(maps, src, replace)
    def __init__(self, *args, **maps):
        self.init(*args, **maps)
    def init(self, *args, **maps):
        pass
    def __call__(self, *args, **maps):
        return self.deal(*args, **maps)
    def deal(self, *args, **maps):
        return None

pass
class EncapeData(Base):
    """
        包含data id对应的配置，配置文件id，配置文件对象
        [object.test, call, ]
    """
    def __init__(self, data, conf, local = False, type = None, src = None, info = None):
        """
            data: 配置数据
            conf: 配置数据对应的配置文件的管理器
            local: 是否是locals的数据（配置文件局部数据）
            type: 配置数据的type字段
            src: 源对象，配置数据生成的对象调用conf获取对象，会有这个字段，目前只有object会放这个字段，其他要么透传要么不传
            info: 额外的调用信息，目前只有object会用到里面的id字段，作为单例额外输入
        """
        if typez(data)==dict:
            pid = xf.g(data, parent=None)
            if pid is not None:
                pedt = conf.get_data(pid, local=True, search_confs = True,src = src, info = info)
                if pedt is None:
                    raise IOCError("unfind parend: "+pid)
                pdt = pedt.data
                if typez(pdt)!=dict:
                    raise IOCError("only dict can be a parent: "+pid)
                data = dict(data)
                self.update_maps(data, pdt, replace=0)
                del data['parent']
        self.data = data
        self.sid = conf.id
        self.src = src
        self.conf = conf
        self.confs = conf.confs
        self.local = local
        if type is None:
            type = conf.confs.get_data_type(data, 0, conf.default_type())
        self.type = type
        self.info = info
    def deal(self, remove = False):
        return self.conf.get(self, src = self.src, info=self.info, remove = remove)


pass



from .. import base
from .. import item
from .. import exp
from ... import file
import json
from . import lr
class PrevStrDeal(lr.LRDeal):
    def prepare(self, mg):
        super().prepare(mg)
        self.label_l2 = mg.like("\\")
        self.label_qt = mg.like('"')
        self.label_et = mg.like("\n")
        self.label_lr = mg.like("\r")
        self.label_nl = mg.like("")
        self.et_in_right = self.right.count(self.label_et)
    def init(self, left = '"', right= '"', single_line = False, note = False, translate = False):
        super().init(left, right, 'str')
        self.single_line = single_line
        self.note = note
        self.translate = translate
    def json_loads(self, s):
        x = s
        cd = None
        if type(x)==bytes:
            x, cd = file.decode_c(x)
        rs = json.loads(x)
        if type(s)==bytes:
            rs = rs.encode(cd)
        return rs
    def do_translate(self, s):
        """
            取巧直接调用json
        """
        qt = self.label_qt
        ql = self.label_l2
        et = self.label_et
        tr = self.label_lr
        nt = self.label_nl
        pt = ql+qt
        arr = s.split(pt)
        arr = [k.replace(qt, pt) for k in arr]
        s = pt.join(arr)
        #s = s.replace(qt, ql+qt)
        s = s.replace(tr, nt)
        arr = s.split(et)
        outs = [self.json_loads(qt+k+qt) for k in arr]
        outs = et.join(outs)
        return outs
    def deal(self, buffer, rst, mg):
        cl = buffer.read(self.ll)
        if cl != self.left:
            return False
        rm = buffer.full().strip()
        buffer.clean2read(self.ll)
        if len(rm)>0 and not self.note:
            if not self.note:
                print("left:", self.left, rm)
                print(f"rst: [{rst}]")
                raise Exception(f"unexcept char before string: {rm}")
            else:
                rst.append(item.Item(rm, type = "", is_val = 0))
        tmp = cl[:0]
        ctmp = tmp[:0]
        do_judge = 1
        mark_et = 0
        mark_l2 = 0
        while True:
            if do_judge and self.right == buffer.rget(self.lr):
                break
            c = buffer.read_cache(1)
            if do_judge and c == self.label_et:
                mark_et += 1
            if len(c)==0:
                if self.single_line and self.note:
                    break
                raise Exception(f"unexcept string end while reading str")
            do_judge = 1
            if c == self.label_l2:
                mark_l2 = 1
                do_judge = 0
                c = buffer.read_cache(1)
                if len(c)==0:
                    raise Exception(f"unexcept string end while reading str")
        data = buffer.full()
        data = data[:-self.lr]
        buffer.clean()
        mark_et -= self.et_in_right
        if self.single_line and mark_et>0:
            print("left:",self.left, "right:", self.right)
            raise Exception(f"contain enter in single line string")
        if self.translate and mark_l2:
            data = self.do_translate(data)
        if self.note:
            return True
        rst.append(item.Item(data, type='str', is_val = 1))
        return True

pass



#coding=utf-8
import os
#列出文件夹下所有文件和文件更新时间
#文件访问处理方法
#建议visit读取文件
#catch处理目录访问异常
class FileDeal:
    def __init__(self, *argv, **maps):
        self.init(*argv, **maps)
    def result(self):
        return None
    def init(self, *argv, **maps):
        pass
    def work(self, *argv, **maps):
        return self.dirs(*argv, **maps)
    def dirs(self, filepath, depth = 0):
        dirs(filepath, self, depth)
        return self.result()
    def visit(self, filepath, isdir, depth):
        return True
    def catch(self, filepath, isdir, depth, exp):
        pass
    def deal(self, filepath, isdir, depth, exp = None):
        if exp is None:
            return self.visit(filepath, isdir, depth)
        else:
            return self.catch(filepath, isdir, depth, exp)
    def __call__(self, *argv, **maps):
        return self.deal(*argv, **maps)

pass
#遍历文件／文件夹filepath
def dirs(filepath, fc=FileDeal(), depth = 0):
    isdir = os.path.isdir(filepath)
    visit = fc(filepath, isdir, depth)
    if isdir and visit:
        try:
            files = os.listdir(filepath)
        except Exception as exp:
            fc(filepath, isdir, depth, exp)
            return
        files = [os.path.join(filepath, file) for file in files]
        [dirs(fp, fc, depth+1) for fp in files]

pass

